@@ -77,5 +77,48 @@ test_that("Link capacity importation works", {
   expect_gt(nrow(input), 0)
   expect_equal(nrow(input) %% (24 * 7 * nweeks), 0)
 })
+
+test_that("readInputTs must work if we change opts$timeIdMin and opts$timeIdMax", {
+  input <- readInputTS(ror = "all", 
+                       showProgress = FALSE, 
+                       timeStep = "hourly",
+                       opts = opts)
+  expect_is(input, "antaresDataTable")
+  
+  sumRorA <- input[ , .(sumProdRor = sum(ror)), by= .(area, tsId)]
+  sumRorA[, ror := sumProdRor]
+  sumRorA <- sumRorA[, ror, by= .(area, tsId)]
+  
+  inputA <- suppressWarnings(readInputTS(ror = "all", 
+                       showProgress = FALSE, 
+                       timeStep = "annual",
+                       opts = opts))
+  
+  inputA[, .(ror) , by= .(area, tsId)]
+  
+  expect_equal(sumRorA, inputA[, .(ror) , by= .(area, tsId)])
+
+  optsNew <- opts
+  optsNew$timeIdMin <- 3500
+  optsNew$timeIdMax <- 4000
+  
+  inputAN <- suppressWarnings(readInputTS(ror = "all",
+                        showProgress = FALSE,
+                        timeStep = "annual",
+                        opts = optsNew))
+  
+  inputH <- readInputTS(ror = "all", 
+                       showProgress = FALSE, 
+                       timeStep = "hourly",
+                       opts = optsNew)
+  
+  sumRorANew <- inputH[ , .(sumProdRor = sum(ror)), by= .(area, tsId)]
+  sumRorANew[, ror := sumProdRor]
+  sumRorANew <- sumRorANew[, ror, by= .(area, tsId)]
+  
+  expect_equal(inputAN[, .( ror) , by= .(area, tsId)], sumRorANew)
+  
+})
+
 }
 })
